@@ -1,7 +1,7 @@
 import json
 from datetime import datetime
 from typing import Union, Dict, List, Awaitable, Optional
-from pysirix.types import Commit, Revision as RevisionType
+from pysirix.types import Commit, Revision as RevisionType, SubtreeRevision
 from json import dumps
 
 from abc import ABC
@@ -13,6 +13,20 @@ from pysirix.sync_client import SyncClient
 from pysirix.types import QueryResult
 
 
+def stringify(v: Union[None, int, str, Dict, List]):
+    return (
+        f'"{v}"'
+        if isinstance(v, str)
+        else "true()"
+        if v is True
+        else "false()"
+        if v is False
+        else "jn:null()"
+        if v is None
+        else v
+    )
+
+
 class JsonStoreBase(ABC):
     def __init__(
         self,
@@ -29,26 +43,29 @@ class JsonStoreBase(ABC):
 
     def insert_one(self, insert_dict: Union[str, Dict]) -> Union[str, Awaitable[str]]:
         """
-        Inserts a single record into the store. New records are added at the head of the store.
+        Inserts a single record into the store. New records are added at the tail of the store.
 
-        :param insert_dict: either a JSON string, or a ``dict`` that can be converted to JSON.
-        :return: a JSON string of the database.
+        :param insert_dict: either a JSON string of a ``dict``, or a ``dict`` that can be converted to JSON.
+        :return: an emtpy ``str`` or an empty ``Awaitable[str]``.
         """
         insert_dict = dumps(insert_dict)
-        return self._client.update(
-            self.db_name,
-            self.db_type,
-            self.name,
-            1,
-            insert_dict,
-            Insert.CHILD,
-            etag=None,
-        )
+        query = f"append json jn:parse('{insert_dict}') into jn:doc('{self.db_name}','{self.name}')"
+        return self._client.post_query({"query": query})
 
-    def insert_many(self, insert_list: Union[str, List[Dict]]):
+    def insert_many(
+        self, insert_list: Union[str, List[Dict]]
+    ) -> Union[str, Awaitable[str]]:
+        """
+        Inserts a list of records into the store. New records are added at the the tail of the store.
+
+        :param insert_list: either a JSON string of ``list`` of ``dict``\s, or a ``list`` that can be converted to JSON
+        :return: a ``str`` "{rest: []}" or an ``Awaitable[str]`` resolving to this string.
+        """
         insert_list = dumps(insert_list)
-        query = f"for $i in {insert_list} return insert json $i into jn:doc({self.db_name},{self.name}) at position 0"
-        # query = "insert json (1,2,3) into [] at position 0"
+        query = (
+            f"let $doc := jn:doc('{self.db_name}','{self.name}')"
+            f"for $i in jn:parse('{insert_list}') return append json $i into $doc"
+        )
         return self._client.post_query({"query": query})
 
     def exists(self) -> Union[bool, Awaitable[bool]]:
@@ -70,42 +87,40 @@ class JsonStoreBase(ABC):
         )
 
     def history(
-        self, node_key: int = 0, revision: bool = True, timestamp: bool = True
-    ) -> Union[List[Commit], List[int], List[str], List[RevisionType]]:
+        self, node_key: int = 0, subtree: bool = True
+    ) -> Union[List[Commit], List[SubtreeRevision], List[RevisionType]]:
         """
         This method returns the history of a resource. If ``node_key`` is not specified,
         it returns the history of the resource (root) itself, in the form of a list of :py:class:`Commit`.
 
         If ``node_key`` is specified, the method returns a list of :py:class:`pysirix.types.Revision`.
-        In such a case, if `timestamp` is specified as ``False``, a list of `int` of revision numbers will be returned.
-        Otherwise, if `revision` is specified as `False`, then a list of `str` of timestamps will be returned.
 
         :param node_key: the root of the subtree whose history should be returned. Defaults to document root.
-        :param revision: whether to return the revision number, when node_key is not document root. Defaults to `True`.
-        :param timestamp: whether to return the revision timestamp, when node_key is not document root.
-                Defaults to ``True``.
-        :return: a list of :py:class:`pysirix.types.Commit` if node_key is not specified. Otherwise, a list of either
-                `int`, `str`, or :py:class:`pysirix.types.Revision`, depending on the specified arguments.
+        :return: a list of :py:class:`pysirix.types.Commit` if node_key is not specified. Otherwise, a list of
+                :py:class:`pysirix.types.Revision`.
         """
         if node_key == 0:
             return self._client.history(self.db_name, self.db_type, self.name)
-        if timestamp:
-            if revision:
-                revision_data = '{"revision": sdb:revision($rev), "timestamp": xs:string(sdb:timestamp($rev))}'
-            else:
-                revision_data = "xs:string(sdb:timestamp($rev))"
+        if subtree:
+            revision_data = '{"revisionNumber": sdb:revision($rev), "revisionTimestamp": xs:string(sdb:timestamp($rev))}'
+            query = (
+                f"let $node := sdb:select-item(., {node_key}) let $result := for $rev in jn:all-times($node)"
+                f" return if (not(exists(jn:previous($rev)))) then {revision_data}"
+                f" else if (sdb:hash($rev) ne sdb:hash(jn:previous($rev))) then {revision_data}"
+                " else () return $result"
+            )
         else:
-            revision_data = "sdb:revision($rev)"
-
-        query = (
-            f"let $node := sdb:select-node(., {node_key}) let $result := for $rev in jn:all-times($node)"
-            f" return if (not(exists(jn:previous($rev)))) then {revision_data}"
-            f" else if (sdb:hash($rev) ne sdb:hash(jn:previous($rev))) then {revision_data}"
-            " else () return $result"
-        )
+            query = f"sdb:item-history(sdb:select-item(., {node_key}))"
         params = {"query": query}
         return self._client.read_resource(self.db_name, self.db_type, self.name, params)
 
+    @staticmethod
+    def _prepare_query_dict(query_dict: Dict) -> str:
+        query_list = []
+        for k, v in query_dict.items():
+            query_list.append(f"deep-equal($i=>{stringify(k)}, {stringify(v)}) and")
+        return " ".join(query_list)[:-4]
+
     def _prepare_find_all(
         self,
         query_dict: Dict,
@@ -126,22 +141,10 @@ class JsonStoreBase(ABC):
             query_list = [
                 f"for $i in bit:array-values(jn:doc('{self.db_name}','{self.name}',{revision})) where"
             ]
-        for k, v in query_dict.items():
-            v = (
-                f'"{v}"'
-                if isinstance(v, str)
-                else "true()"
-                if v is True
-                else "false()"
-                if v is False
-                else "jn:null()"
-                if v is None
-                else v
-            )
-            query_list.append(f"deep-equal($i=>{k}, {v}) and")
+        query_list.append(self._prepare_query_dict(query_dict))
         query_string = " ".join(
             [
-                " ".join(query_list)[:-4],
+                *query_list,
                 "return {$i,'nodeKey': sdb:nodekey($i)}" if node_key else "return {$i}",
             ]
         )
@@ -168,13 +171,53 @@ class JsonStoreBase(ABC):
     ):
         raise NotImplementedError()
 
+    def update_by_key(
+        self, node_key: int, field: str, value: Union[int, str, List, Dict]
+    ):
+        query = (
+            f"let $obj := sdb:select-item(jn:doc('{self.db_name}','{self.name}'),{node_key}) "
+            f"return replace json value of $obj=>{stringify(field)} with {stringify(value)}"
+        )
+        return self._client.post_query({"query": query})
+
+    def update_many(
+        self, query_dict: Dict, field: str, value: Union[None, int, str, List, Dict]
+    ):
+        query = (
+            f"for $i in jn:doc('{self.db_name}','{self.name}') where {self._prepare_query_dict(query_dict)}"
+            f" return replace json value of $i=>{stringify(field)} with {stringify(value)}"
+        )
+        return self._client.post_query({"query": query})
+
+    def delete_field_by_key(self, node_key: int, field: str):
+        query = (
+            f"let $obj := sdb:select-item(jn:doc('{self.db_name}','{self.name}'),{node_key})"
+            f" return delete json $obj=>{stringify(field)}"
+        )
+        return self._client.post_query({"query": query})
+
+    def delete_field(self, query_dict: Dict, field: str):
+        query = (
+            f"for $i in jn:doc('{self.db_name}','{self.name}') where {self._prepare_query_dict(query_dict)}"
+            f" return delete json $i=>{stringify(field)}"
+        )
+        return self._client.post_query({"query": query})
+
+    def delete_records(self, query_dict: Dict):
+        query = (
+            f"let $doc := jn:doc('{self.db_name}','{self.name}')"
+            f" let $m := for $i at $pos in $doc where {self._prepare_query_dict(query_dict)} return $pos - 1"
+            " for $i in $m order by $i descending return delete json $doc[[$i]]"
+        )
+        return self._client.post_query({"query": query})
+
     def find_one(
         self,
         query_dict: Dict,
         projection: List[str] = None,
         revision: Revision = None,
         node_key=True,
-    ) -> Dict[str, List[QueryResult]]:
+    ) -> List[QueryResult]:
         return self.find_all(
             query_dict,
             projection,
@@ -190,21 +233,11 @@ class JsonStoreSync(JsonStoreBase):
     This class is a convenient abstraction over the resource entities exposed by SirixDB.
     As such, there is no JsonStore on the SirixDB server, only the underlying resource is stored.
 
-    This class is for storing many distinct, json objects in a single resource,
-    where the objects storing data of similar type. As such, an object is an
-    abstraction similar to a row in a relational database. The entirety of the
-    store parallels a table in a relational database.
+    This class is for storing many distinct, JSON objects in a single resource, where the objects/records
+    store data of similar type. As such, it's usage parallels a that of a document store, and an object is
+    an abstraction similar to a single document in such a store.
     """
 
-    def __init__(
-        self,
-        db_name: str,
-        name: str,
-        client: Union[SyncClient, AsyncClient],
-        auth: Auth,
-    ):
-        super().__init__(db_name, name, client, auth)
-
     def find_all(
         self,
         query_dict: Dict,
@@ -213,7 +246,7 @@ class JsonStoreSync(JsonStoreBase):
         node_key=True,
         start_result_index: Optional[int] = None,
         end_result_index: Optional[int] = None,
-    ) -> Dict[str, List[QueryResult]]:
+    ) -> List[QueryResult]:
         """
         Finds and returns all records where the values of ``query_dict`` match the
         corresponding values the record.
@@ -231,7 +264,7 @@ class JsonStoreSync(JsonStoreBase):
         :param revision: the revision to search, defaults to latest.
         :param start_result_index: index of first result to return.
         :param end_result_index: index of last result to return.
-        :return: a ``dict`` with a field ``rest``, containing a ``list`` of ``dict`` records matching the query.
+        :return: a ``list`` of :py:class:`QueryResult` records matching the query.
         """
         params = self._prepare_find_all(
             query_dict,
@@ -242,7 +275,7 @@ class JsonStoreSync(JsonStoreBase):
             end_result_index,
         )
         result = self._client.post_query(params)
-        return json.loads(result)
+        return json.loads(result)["rest"]
 
 
 class JsonStoreAsync(JsonStoreBase):
@@ -251,15 +284,6 @@ class JsonStoreAsync(JsonStoreBase):
     with async support.
     """
 
-    def __init__(
-        self,
-        db_name: str,
-        name: str,
-        client: Union[SyncClient, AsyncClient],
-        auth: Auth,
-    ):
-        super().__init__(db_name, name, client, auth)
-
     async def find_all(
         self,
         query_dict: Dict,
@@ -268,7 +292,7 @@ class JsonStoreAsync(JsonStoreBase):
         node_key=True,
         start_result_index: Optional[int] = None,
         end_result_index: Optional[int] = None,
-    ) -> Awaitable[Dict[str, List[QueryResult]]]:
+    ) -> Awaitable[List[QueryResult]]:
         """
         Finds and returns all records where the values of ``query_dict`` match the
         corresponding values the record.
@@ -286,7 +310,7 @@ class JsonStoreAsync(JsonStoreBase):
         :param revision: the revision to search, defaults to latest.
         :param start_result_index: index of first result to return.
         :param end_result_index: index of last result to return.
-        :return: a ``dict`` with a field ``rest``, containing a ``list`` of ``dict`` records matching the query.
+        :return: an ``Awaitable`` ``list`` of :py:class:`QueryResult` records matching the query.
         """
         params = self._prepare_find_all(
             query_dict,
@@ -297,4 +321,4 @@ class JsonStoreAsync(JsonStoreBase):
             end_result_index,
         )
         result = await self._client.post_query(params)
-        return json.loads(result)
+        return json.loads(result)["rest"]
